@@ -37,6 +37,9 @@ main (int argc, char **argv)
     cout << "This is the integer you input: " << 
       args_info.int_opt_arg << "." << endl;
 
+  if (args_info.flag_opt_given)
+    cout << "The flag option was given!" << endl;
+
   cout << "The flag is " << ( args_info.flag_opt_flag ? "on" : "off" ) <<
     "." << endl ;
 
