@@ -32,6 +32,8 @@ extern struct gengetopt_option * gengetopt_options;
 extern char * gengetopt_package;
 extern char * gengetopt_version;
 extern char * gengetopt_purpose;
+extern int gengetopt_strdup_text_length;
+extern char *gengetopt_strdup_text[];
 
 static int tab_indentation ; /* tab indentation level */
 static int handle_error; /* should I handle errors, by exit(1) */
@@ -50,14 +52,6 @@ static void generate_error_handle();
 
 static char *parser_function_name;
 
-long
-gengetopt_strlen (char *s)
-{
-  char * p = s;
-  while (*p) p++;
-  return p - s;
-}
-
 char *
 create_filename (char *name, char *ext)
 {
@@ -100,6 +94,7 @@ do_check_option_given (struct gengetopt_option *opt)
         check_option_given (opt->var_arg, opt->long_opt, opt->short_opt);
         break;
 	case ARG_FLAG:
+        check_option_given (opt->var_arg, opt->long_opt, opt->short_opt);
         indent ();
         printf ("%s->%s_flag = !(%s->%s_flag);\n",
                 ARGS_STRUCT,
@@ -486,11 +481,11 @@ header_filename);
   if ( unamed_options )
       printf (" [FILES]...");
 
-  printf ("\\n\\\n");
+  printf ("\\n\", PACKAGE);\n");
   /* calculate columns */
   max_long = max_short = 0;
   foropt {
-	w = 3 + gengetopt_strlen (opt->long_opt);
+	w = 3 + strlen (opt->long_opt);
 	if (opt->type == ARG_FLAG || opt->type == ARG_NO)
 	{
 		if (w > max_long) max_long = w;
@@ -498,52 +493,55 @@ header_filename);
 	}
 	else
 	{
-		w += gengetopt_strlen (arg_names[opt->type]);
+		w += strlen (arg_names[opt->type]);
 		if (w > max_long) max_long = w;
-		w = (3 + gengetopt_strlen (arg_names[opt->type]));
+		w = (3 + strlen (arg_names[opt->type]));
 		if (w > max_short) max_short = w;
 	}
   }
   /* print justified options */
   foropt
-	if (opt->type == ARG_FLAG || opt->type == ARG_NO)
-	{
-        if (opt->short_opt) printf ("   -%c", opt->short_opt);
-        else                printf ("     ");
-		for (w = 2; w < max_short; w++) printf (" ");
-   		printf ("  --%s", opt->long_opt);
-		for (w = 2+gengetopt_strlen(opt->long_opt); w < max_long; w++)
-			printf (" ");
-		printf ("  %s", opt->desc);
-		if (opt->type == ARG_FLAG)
- 	    {
-		    if (opt->flagstat)
-                printf (" (default=on)");
-            else
-		       printf (" (default=off)");
-        }
-		printf ("\\n\\\n");
-	}
+  {
+    printf ("  printf(\"");
+    if (opt->type == ARG_FLAG || opt->type == ARG_NO)
+    {
+      if (opt->short_opt) printf ("   -%c", opt->short_opt);
+      else                printf ("     ");
+      for (w = 2; w < max_short; w++) printf (" ");
+      printf ("  --%s", opt->long_opt);
+      for (w = 2+strlen(opt->long_opt); w < max_long; w++)
+	printf (" ");
+      printf ("  %s", opt->desc);
+      if (opt->type == ARG_FLAG)
+      {
+	if (opt->flagstat)
+	  printf (" (default=on)");
 	else
-	{
-		if (opt->short_opt)
-            printf ("   -%c%s", opt->short_opt, arg_names[opt->type]);
-        else
-        {
-            int type_len = gengetopt_strlen(arg_names[opt->type]);
+	  printf (" (default=off)");
+      }
+      printf ("\\n\");\n");
+    }
+    else
+    {
+      if (opt->short_opt)
+	printf ("   -%c%s", opt->short_opt, arg_names[opt->type]);
+      else
+      {
+	int type_len = strlen(arg_names[opt->type]);
 
-            printf ("      ");
-            for (w = 1; w < type_len; w++) printf (" ");
-        }
-		for (w = 2+gengetopt_strlen(arg_names[opt->type]); w < max_short; w++)
-			printf (" ");
-		printf ("  --%s=%s", opt->long_opt, arg_names[opt->type]);
-		for (w = 3+gengetopt_strlen(opt->long_opt)+
-		     gengetopt_strlen(arg_names[opt->type]); w < max_long; w++)
-			printf (" ");
-   		printf ("  %s\\n\\\n", opt->desc);
-	}
-  printf ("\", PACKAGE);\n}\n\n\n");
+	printf ("      ");
+	for (w = 1; w < type_len; w++) printf (" ");
+      }
+      for (w = 2+strlen(arg_names[opt->type]); w < max_short; w++)
+	printf (" ");
+      printf ("  --%s=%s", opt->long_opt, arg_names[opt->type]);
+      for (w = 3+strlen(opt->long_opt)+
+	     strlen(arg_names[opt->type]); w < max_long; w++)
+	printf (" ");
+      printf ("  %s\\n\");\n", opt->desc);
+    }
+  }
+  printf ("}\n\n\n");
 
   if (! generate_strdup)
     {
@@ -554,22 +552,12 @@ header_filename);
 	}
     }
 
-  if (generate_strdup)
-    printf (
-            "static char *\n"
-            "gengetopt_strdup (char * s)\n"
-            "{\n"
-            "  char * n, * pn, * ps = s;\n"
-            "  while (*ps) ps++;\n"
-            "  n = (char *) malloc (1 + ps - s);\n"
-            "  if (n != NULL)\n"
-            "    {\n"
-            "      for (ps=s,pn=n; *ps; ps++,pn++)\n"
-            "        *pn = *ps;\n"
-            "      *pn = 0;\n"
-            "    }\n"
-            "  return n;\n"
-            "}\n\n\n");
+  if (generate_strdup) 
+    {
+      for (i = 1; i <= gengetopt_strdup_text_length; ++i)
+        printf ("%s\n", gengetopt_strdup_text[i]);
+      printf ("\n");
+    }
   
   printf (
 "int\n"
